from cStringIO import StringIO
import re
import tempfile
from urlparse import urlparse

from django.db import models
from django.conf import settings
from django.contrib.auth.models import User
from django.core.files import File
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from PIL import Image

from autoslug import AutoSlugField
from photologue.models import ImageModel, PHOTOLOGUE_DIR
from taggit.managers import TaggableManager

from utils.openanything import fetch


if getattr(settings, 'METAIMAGE_MAX_REMOTE_IMAGE_SIZE', False):
    MAX_REMOTE_IMAGE_SIZE = settings.METAIMAGE_MAX_REMOTE_IMAGE_SIZE
else:
    MAX_REMOTE_IMAGE_SIZE = 1048576  # 1 MB

# The METAIMAGE_DIR parameter is referenced in tests.py when cleaning
# up after unit-tests, otherwise old test images litter the directory:
#
# Note that PHOTOLOGUE_DIR is the "top-level" directory which holds
# several others: PHOTOLOGUE_PATH is where original-image files are
# stored; temp/ is where uploads are held; samples/, watermarks/ are
# two more.  So to customize, both PHOTOLOGUE_DIR and PHOTOLOGUE_PATH
# (which can be a function) should be in settings.py.
if getattr(settings, 'PHOTOLOGUE_DIR', False):
    METAIMAGE_DIR = settings.PHOTOLOGUE_DIR
else:
    METAIMAGE_DIR = PHOTOLOGUE_DIR


PRIVACY_CHOICES = (
    (1, _('Public')),
    (2, _('Friends')),
    (3, _('Private')),
    )


class MetaImageException(Exception):
    pass


class MetaImageUnableToRetrieveSourceURL(MetaImageException):
    pass


class MetaImageSourceURLNotAnImage(MetaImageException):
    pass


class BaseModel(models.Model):
    """
    An abstract base class that provides standard fields for every
    model.
    """
    admin_notes = models.TextField(blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True, editable=False)
    creator = models.ForeignKey(
        User,
        editable=False,
        related_name='%(class)s_creator')
    updated = models.DateTimeField(auto_now=True, editable=False, null=True)
    updater = models.ForeignKey(
        User,
        editable=False,
        null=True,
        related_name='%(class)s_updater')
    is_active = models.BooleanField(default=True)

    class Meta:
        abstract = True


class ImageSet(BaseModel):
    """
    A set of images, akin to a photo-album.
    """
    name = models.CharField(_('name'), max_length=200)
    description = models.TextField(_('description'))
    privacy = models.IntegerField(
        _('privacy'),
        choices=PRIVACY_CHOICES, default=1)
    tags = TaggableManager()


class MetaImage(ImageModel, BaseModel):
    """
    An image with its useful details.  Parallels photologue's Photo
    class, but with different functionality.

    Note that ImageModel from photologue.models is an abstract model
    too, so there's no one-to-one joining going on here, thankfully.
    As to what the ImageModel base class provides, view the ImageModel
    source at,
    http://code.google.com/p/django-photologue/source/browse/trunk/photologue/models.py

    The key ImageModel attributes include:
    - image, the ImageField
    - date_taken
    - view_count
    - crop_from, default 'center'
    - effect, e.g. WaterMark, PhotoEffect

    Useful methods from ImageModel include:
    - image_filename()
    - get_<photosize>_url(), get_<photosize>_filename() where
      <photosize> is already defined. Note that django-metaimage
      defines some starting PhotoSize instances in
      fixtures/initial_data.json such as square25, square50, etc.
    """
    SAFETY_LEVEL = (
        (1, _('Safe')),
        (2, _('Not Safe')),
    )
    title = models.CharField(_('title'), max_length=200)
    # With AutoSlugField, editable should be set to True to avoid
    # problems with Django's admin form generation - though in
    # practice slugs should not be manually edited:
    slug = AutoSlugField(
        max_length=100, editable=True, populate_from='title', unique=True,
        help_text='In general do NOT edit slugs manually.')
    caption = models.TextField(_('caption'), blank=True)
    # The source of a MetaImage may be from another site; if so,
    # provide URL here.
    source_url = models.URLField(
        _('source URL'),
        blank=True, null=True,
        verify_exists=True,
        help_text='The external URL where the image originated from.')
    source_note = models.CharField(
        _('source note'),
        max_length=500,
        blank=True, null=True,
        help_text='Attribution text, copyright notices, permissions, etc.')
    safetylevel = models.IntegerField(
        _('safetylevel'), choices=SAFETY_LEVEL, default=1)
    privacy = models.IntegerField(
        _('privacy'), choices=PRIVACY_CHOICES, default=1)
    imageset = models.ManyToManyField(
        ImageSet, blank=True, null=True, verbose_name=_('image set'))
    tags = TaggableManager(blank=True)  # taggit has blank=False by default.

    class Meta:
        verbose_name = 'MetaImage'

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ("metaimage_details", [self.pk])

    def is_str_an_image(self, the_string):
        # Verify that the data is, in fact, an image.
        assert isinstance(the_string, str)
        the_image = Image.open(StringIO(the_string))
        try:
            the_image.verify()
            return True
        except Exception:
            return False

    def save(self, *args, **kwargs):
        """
        MetaImage.save transparently fetches remote images to local
        storage - in case the remote image ever changes location or
        disappears, you'll still have it.

        For the other major use-case of server-side generated images,
        pass in the raw image data as a string with
        save(image_data=the_image_as_string).
        """
        raw_image_data = None
        local_img_filename = None
        # There are three cases of how we get image data: 1) uploaded,
        # 2) remote image URL given, or 3) the raw image string is
        # passed in.  Cases 2) and 3) are handled below:
        if getattr(self, 'source_url') and not getattr(self, 'image'):
            # Download remote data, synchronously for now:
            image_data_dct = fetch(
                self.source_url,
                max_size=MAX_REMOTE_IMAGE_SIZE)
            if not image_data_dct:
                raise MetaImageUnableToRetrieveSourceURL
            raw_image_data = image_data_dct['data']
            local_img_filename = self.generate_filename_from_url()
        elif 'image_data' in kwargs and not self.source_url and not self.image:
            # Alternatively, raw image data (as a string) was given:
            raw_image_data = kwargs.pop('image_data')
            local_img_filename = self.generate_filename_from_data(
                raw_image_data)
        # Create and save the remote/given image if needed:
        if raw_image_data and local_img_filename:
            assert self.is_str_an_image(raw_image_data)
            # Django's save() method must receive a Django File
            # object, which in turn, must be wrapped around a real
            # file (not StringIO) to work. Thus use tempfile:
            tmp_file = tempfile.NamedTemporaryFile()
            tmp_file.write(raw_image_data)
            self.image.save(local_img_filename, File(tmp_file))
        if self.creator and not self.updater:
            self.updater = self.creator
        super(MetaImage, self).save(*args, **kwargs)

    def generate_filename_from_url(self, the_url=None):
        """
        When a remote image is fetched, the URL will be unique, and
        that handily provides the basis for a unique filename too.
        """
        if not the_url:
            the_url = self.source_url
        parsed_url = urlparse(the_url)
        non_alphanum_regex = re.compile(r'[^a-zA-Z0-9\-\.]')
        raw_filename = parsed_url.netloc.lower() + parsed_url.path
        the_filename = non_alphanum_regex.sub('_', raw_filename)
        return the_filename

    def generate_filename_from_data(self, the_data):
        # Assume only PNG files for now:
        return '%s.png' % hash(the_data)

    def get_public_status(self):
        """
        Public images can be displayed in default views.
        """
        return bool(self.privacy == 1)

    is_public = property(get_public_status)

    def render(self, the_size='width500', linked=False):
        """
        Returns the HTML to display this MetaImage instance.
        """
        get_url_method_name = 'get_%s_url' % the_size
        mimage_url = getattr(self, get_url_method_name)()
        get_size_method_name = 'get_%s_size' % the_size
        the_width, the_height = getattr(self, get_size_method_name)()
        img_html = mark_safe(
            '<img src="%s" height="%s" width="%s" alt="%s">'
            % (mimage_url, the_height, the_width, str(self.title)))
        if linked:
            return mark_safe(
                '<a href="%s" class="invisible">%s</a>'
                % (self.get_absolute_url(), img_html))
        else:
            return img_html

    def render_linked(self, the_size='width500'):
        return self.render(the_size, linked=True)

    def render_thumbnail_linked(self):
        return self.render_linked(the_size='square25')



DATABASES = {
    'default': {
        'ENGINE': 'sqlite3',
        'NAME': '/tmp/metaimage.db'}
    }

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'metaimage',
    'photologue',
    'taggit',
    'uni_form')

METAIMAGE_MAX_REMOTE_IMAGE_SIZE = 1048576  # 1 MB

ROOT_URLCONF = 'metaimage.urls'



import shutil

from django.contrib.auth.models import User
from django.test import TestCase
from django.test.client import Client

from metaimage.forms import MetaImageUploadForm, MetaImageEditForm
from metaimage.models import MetaImage, METAIMAGE_DIR


class TestMetaImage(TestCase):
    """
    Testing of the MetaImage model class.
    """

    def setUp(self):
        self.foo = User.objects.create_user('foo', 'foo@test.com', 'bar')
        self.remote_img_url = 'http://media.djangoproject.com/img/site/hdr_logo.gif'

    def test_metaimage_model(self):
        """
        Test fetching and saving a remote image as a new MetaImage
        instance.  In lieu of multiple tests with multiple network
        requests, I'm consolidating several tests of the MetaImage
        model into this single testcase.

        MetaImage wraps around photologue.ImageModel, which wraps
        around Django's ImageField; this simple test actually hits a
        lot of layers.
        """
        test_metaimage = MetaImage(
            title='Django logo',
            source_url=self.remote_img_url,
            source_note='The logo of the Django project.',
            creator=self.foo,
            )
        test_metaimage.save()
        the_metaimage = MetaImage.objects.get(slug='django-logo')
        assert the_metaimage.image is not None  # Image was downloaded OK.
        assert the_metaimage.is_public
        assert the_metaimage.updater == self.foo
        assert the_metaimage.slug == 'django-logo'
        assert (the_metaimage.image.name
                == 'photologue/photos/media.djangoproject.com_img_site_hdr_logo.gif')
        assert (the_metaimage.render()
                == '<img src="photologue/photos/cache/media.djangoproject.com_img_site_hdr_logo_width500.gif" height="41" width="117" alt="Django logo">')
        # As of Django 1.2.5, the image file is NOT removed when the
        # database object is deleted.
        the_metaimage.delete()

    def tearDown(self):
        # In order that image filenames are repeatable, it's necessary
        # to delete the directory where they're stored between test
        # runs.
        shutil.rmtree(METAIMAGE_DIR)


class TestViews(TestCase):
    """
    Tests of metaimage/views.py module.
    """

    def setUp(self):
        self.client = Client()
        self.foo = User.objects.create_user('foo', 'foo@test.com', 'bar')
        self.client.login(username='foo', password='bar')
        self.remote_img_url = 'http://www.google.com/images/nav_logo36.png'
        test_metaimage = MetaImage(
            title='Google small logo',
            source_url=self.remote_img_url,
            source_note='The Google logo, for testing.',
            creator=self.foo,
            )
        test_metaimage.save()
        self.test_image = test_metaimage

    def test_show_metaimages(self):
        response = self.client.get('')
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual(len(response.context['metaimages']), 1)

    def test_metaimage_details(self):
        response = self.client.get('/details/1/')
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual(response.context['the_metaimage'].slug,
                             'google-small-logo')

    def test_upload_metaimage(self):
        response = self.client.get('/upload/')
        self.failUnlessEqual(response.status_code, 200)
        self.assertEqual(isinstance(response.context['metaimage_form'],
                                    MetaImageUploadForm),
                         True)

    def test_upload_metaimage_post(self):
        upload_dct = {
            'source_url': 'http://upload.wikimedia.org/wikipedia/en/b/bc/Wiki.png',
            'title': 'Wikipedia logo, English',
            'safetylevel': 1,
            'privacy': 1,
            'action': 'upload'}
        response = self.client.post('/upload/', upload_dct)
        self.assertRedirects(response, '/details/2/')

    def test_your_metaimages(self):
        response = self.client.get('/user/foo/')
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual(len(response.context['metaimages']), 1)
        self.assertContains(response, 'Google small logo')

    def test_show_user_metaimages(self):
        monty = User.objects.create_user('monty', 'monty@test.com', 'python')
        response = self.client.get('/user/monty/')
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual(len(response.context['metaimages']), 0)
        self.failUnlessEqual(response.context['the_user'].username,
                             monty.username)

    def test_edit_metaimage(self):
        response = self.client.get('/edit/1/')
        self.failUnlessEqual(response.status_code, 200)
        self.assertEqual(isinstance(response.context['metaimage_form'],
                                    MetaImageEditForm),
                         True)
        self.assertEqual(response.context['metaimage'].source_url,
                         self.test_image.source_url)

    def test_edit_metaimage_disallowed(self):
        User.objects.create_user('monty', 'monty@test.com', 'python')
        new_client = Client()
        new_client.login(username='monty', password='python')
        # Unfortunately the code below does not work; within the
        # edit_metaimage view during test-running, a crash arises when
        # the Django message_set is used, with a completely cryptic
        # error message, "InterfaceError: Error binding parameter 1 -
        # probably unsupported type."  Tried adding the Django message
        # settings to testsettings.py but no luck.  Manual testing via
        # browser shows this works as expected.  2/2011
        #
        #response = new_client.get('/edit/1/')
        #self.assertRedirects(response, '/details/1/')

    def test_edit_metaimage_ok(self):
        edit_dct = {
            'title': self.test_image.title,
            'privacy': self.test_image.privacy,
            'safetylevel': self.test_image.safetylevel,
            'caption': 'Behold the Google logo!',
            'action': 'update'}
        response = self.client.post('/edit/1/', edit_dct)
        self.assertRedirects(response, '/details/1/')

    def test_destroy_metaimage(self):
        delete_dct = {
            'action': 'delete'}
        response = self.client.post('/destroy/1/', delete_dct)
        self.assertRedirects(response, '/yours/')

    def tearDown(self):
        shutil.rmtree(METAIMAGE_DIR)



import re

from django import template
#from photologue.utils import EXIF


register = template.Library()


@register.tag(name="print_exif")
def do_print_exif(parser, token):
    try:
        tag_name, exif = token.contents.split()
    except ValueError:
        msg = '%r tag requires a single argument' % token.contents[0]
        raise template.TemplateSyntaxError(msg)
    exif = parser.compile_filter(exif)
    return PrintExifNode(exif)


class PrintExifNode(template.Node):

    def __init__(self, exif):
        self.exif = exif

    def render(self, context):
        try:
            exif = unicode(self.exif.resolve(context, True))
        except template.VariableDoesNotExist:
            exif = u''
        EXPR = "'(?P<key>[^:]*)'\:(?P<value>[^,]*),"
        expr = re.compile(EXPR)
        msg = "<table>"
        for i in expr.findall(exif):
            msg += "<tr><td>%s</td><td>%s</td></tr>" % (i[0], i[1])
        msg += "</table>"
        return u'<div id="exif">%s</div>' % (msg)



from django.conf.urls.defaults import patterns, url, handler404, handler500


urlpatterns = patterns('',
    url(r'^$', 'metaimage.views.show_metaimages',
        name="show_metaimages"),
    url(r'^details/(?P<id>\d+)/$', 'metaimage.views.metaimage_details',
        name="metaimage_details"),
    url(r'^edit/(?P<id>\d+)/$', 'metaimage.views.edit_metaimage',
        name='edit_metaimage'),
    url(r'^upload/$', 'metaimage.views.upload_metaimage',
        name="upload_metaimage"),
    url(r'^yours/$', 'metaimage.views.your_metaimages',
        name='your_metaimages'),
    url(r'^user/(?P<username>[\w]+)/$', 'metaimage.views.show_user_metaimages',
        name='show_user_metaimages'),
    url(r'^destroy/(?P<id>\d+)/$', 'metaimage.views.destroy_metaimage',
        name='destroy_metaimage'),
)



from django.contrib import admin

from metaimage.models import MetaImage


class BaseModelAdmin(admin.ModelAdmin):
    """
    Because the sage.models relies on the BaseModel abstract class,
    Django's admin UI does not automagically pick up the
    BaseModel.creator and BaseModel.updater fields; in particular, the
    creator field is required.  So create an inheritable save_model()
    method that takes care of that.
    """
    def save_model(self, request, obj, form, change):
        if not change:
            obj.creator = request.user
        else:
            obj.updater = request.user
        obj.save()


class MetaImageAdmin(BaseModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('image', 'crop_from', 'effect', 'title', 'slug', 'caption', 'source_url', 'source_note', 'safetylevel', 'privacy', 'tags', 'imageset', 'admin_notes', 'is_active')
            }),
        )
    list_display = ('title', 'slug', 'caption', 'creator', 'created', 'is_public', 'safetylevel', 'the_tags')
    prepopulated_fields = {"slug": ("title",)}

    def the_tags(self, obj):
        """
        This workaround of m2m limitation of django-taggit is from
        https://github.com/alex/django-taggit/issues/46
        """
        return "%s" % (obj.tags.all(), )
    the_tags.short_description = 'Tags'

admin.site.register(MetaImage, MetaImageAdmin)



from django.contrib.auth.models import User
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import Http404
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from metaimage.models import MetaImage, PRIVACY_CHOICES
from metaimage.forms import MetaImageUploadForm, MetaImageEditForm


assert PRIVACY_CHOICES[0][0] == 1
assert PRIVACY_CHOICES[0][1] == 'Public'
PUBLIC_SETTING = PRIVACY_CHOICES[0][0]


@login_required
def show_metaimages(request, template_name="metaimage/latest.html"):
    """
    For an authenticated user only, show public and private images -
    most recent first.
    """
    metaimages = MetaImage.objects.filter(
        Q(privacy=PUBLIC_SETTING)
        | Q(privacy__gt=PUBLIC_SETTING, creator=request.user)
        ).order_by("-created")
    return render_to_response(
        template_name,
        {"metaimages": metaimages},
        context_instance=RequestContext(request))


@login_required
def metaimage_details(request, id, template_name="metaimage/details.html"):
    """
    Show details for a MetaImage instance.
    """
    the_metaimage = get_object_or_404(MetaImage, id=id)
    # Private images can only be seen by their creator:
    if not the_metaimage.privacy==PUBLIC_SETTING and the_metaimage.creator != request.user:
        raise Http404
    is_mine = bool(the_metaimage.creator == request.user)
    if is_mine:
        # Could provide extra editing options here for the image's
        # creator, but leaving blank for now.
        pass
    return render_to_response(
        template_name,
        {"the_metaimage": the_metaimage},
        context_instance=RequestContext(request))


@login_required
def upload_metaimage(request, form_class=MetaImageUploadForm, template_name="metaimage/upload.html"):
    """
    Show and process upload form for a MetaImage.
    """
    metaimage_form = form_class(user=request.user)
    if request.method == 'POST' and request.POST.get("action") == "upload":
        metaimage_form = form_class(request.user, request.POST, request.FILES)
        # These two print statements below were handy in debugging
        # unit tests:
        # print metaimage_form.is_valid()
        # print metaimage_form.errors
        if metaimage_form.is_valid():
            metaimage = metaimage_form.save()
            request.user.message_set.create(
                message=_("Successfully uploaded image '%s'.")
                % metaimage.title)
            return HttpResponseRedirect(
                reverse('metaimage_details', args=(metaimage.id,)))
    return render_to_response(
        template_name,
        {"metaimage_form": metaimage_form},
        context_instance=RequestContext(request))


@login_required
def your_metaimages(request, template_name="metaimage/latest.html"):
    """
    Show MetaImages belonging to the currently authenticated user.
    """
    metaimages = MetaImage.objects.filter(
        creator=request.user).order_by("-created")
    return render_to_response(
        template_name,
        {"metaimages": metaimages,
         "page_title": _("Your Images")},
        context_instance=RequestContext(request))


@login_required
def show_user_metaimages(request, username, template_name="metaimage/latest.html"):
    """
    Get a given user's public images, display them.
    """
    the_user = get_object_or_404(User, username=username)
    metaimages = MetaImage.objects.filter(
        creator=the_user, privacy=PUBLIC_SETTING).order_by("-created")
    t_dict = {
        "metaimages": metaimages,
        "page_title": _("Images by %s" % the_user.username),
        "the_user": the_user}
    return render_to_response(
        template_name,
        t_dict,
        context_instance=RequestContext(request))


@login_required
def edit_metaimage(request, id, form_class=MetaImageEditForm, template_name="metaimage/edit.html"):
    metaimage = get_object_or_404(MetaImage, id=id)
    if metaimage.creator != request.user:
        request.user.message_set.create(
            message=_("You can't edit images that aren't yours."))
        return HttpResponseRedirect(
            reverse('metaimage_details', args=(metaimage.id,)))
    metaimage_form = form_class(instance=metaimage)
    if request.method == "POST" and request.POST["action"] == "update":
        metaimage_form = form_class(
            request.user,
            request.POST,
            instance=metaimage)
        if metaimage_form.is_valid():
            metaimage = metaimage_form.save()
            request.user.message_set.create(
                message=_("Successfully updated image '%s'.") % metaimage.title)
            return HttpResponseRedirect(
                reverse('metaimage_details', args=(metaimage.id,)))
    t_dict = {
        "metaimage_form": metaimage_form,
        "metaimage": metaimage}
    return render_to_response(
        template_name,
        t_dict,
        context_instance=RequestContext(request))


@login_required
def destroy_metaimage(request, id):
    # Currently, 2/2011, none of the included MetaImage templates
    # POSTs to this view.
    metaimage = MetaImage.objects.get(pk=id)
    if metaimage.creator != request.user:
        request.user.message_set.create(
            message=_("You can't delete images that aren't yours."))
        return HttpResponseRedirect(reverse("your_metaimages"))
    if request.method == "POST" and request.POST["action"] == "delete":
        title = metaimage.title
        metaimage.delete()
        request.user.message_set.create(
            message=_("Successfully deleted image '%s'.") % title)
    return HttpResponseRedirect(reverse("your_metaimages"))



from django import forms

from metaimage.models import MetaImage


# These are the MetaImage (and base-class ImageModel) fields that
# users should generally not edit directly:
model_fields_to_exclude = (
    'admin_notes',
    'creator',
    'is_active',
    'slug',
    'imageset',
    'effect',
    'crop_from')

model_fields_to_include = (
    'image',
    'source_url',
    'title',
    'caption',
    'privacy',
    'source_note',
    'safetylevel',
    'tags')


class OptionalImageField(forms.ImageField):
    def __init__(self, *args, **kwargs):
        super(forms.ImageField, self).__init__(*args, **kwargs)
        self.required = False


class MetaImageUploadForm(forms.ModelForm):
    image = OptionalImageField()

    class Meta:
        model = MetaImage
        exclude = model_fields_to_exclude
        fields = model_fields_to_include

    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(forms.ModelForm, self).__init__(*args, **kwargs)

    def clean(self):
        """
        The key inter/multi-fields condition to check is: if a local
        image file is not given, then a source_url must be provided.
        MetaImage.save() will then try to grab that remote image, and
        at that point set the image attribute to a new file with that
        remote image data.
        """
        c_data = self.cleaned_data
        local_image = c_data.get('image')
        remote_image_url = c_data.get('source_url')
        if not local_image and not remote_image_url:
            raise forms.ValidationError(
                'Either a local image, or a remote image URL, must be given.')
        return c_data

    def save(self, *args, **kwargs):
        self.instance.creator = self.user
        self.instance.updater = self.user
        return super(MetaImageUploadForm, self).save(*args, **kwargs)


class MetaImageEditForm(forms.ModelForm):
    class Meta:
        model = MetaImage
        fields = model_fields_to_include = (
            'title',
            'caption',
            'privacy',
            'source_note',
            'safetylevel',
            'tags')

    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(forms.ModelForm, self).__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        self.instance.updater = self.user
        return super(MetaImageEditForm, self).save(*args, **kwargs)



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='django-metaimage',
    version='0.4',
    url='https://github.com/limist/django-metaimage',
    license='GPLv3',
    description='Wrapper around django-photologue dealing w/ remote images.',
    long_description=read('README'),
    author='Kai Wu',
    author_email='k@limist.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=[
        'setuptools',
        'Django',
        'django-autoslug',
        'django-photologue',
        'django-taggit',
        'PIL'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP']
    )



#!/usr/bin/env python

"""Utility functions to open URLs are here, particularly to access web
services like Amazon API via REST format.

Code originally from Mark Pilgrim's book, Dive Into Python.
"""

from cStringIO import StringIO
import gzip
import socket
from time import sleep
import urllib2 
import urlparse


USER_AGENT = 'Test'
FETCH_RETRY_NUM = 5
FETCH_RETRY_DELAY = 1.2  # seconds
FETCH_MAX_SIZE = 1048576  # 1 MB
# Set global timeout
timeout = 5  # seconds
socket.setdefaulttimeout(timeout)


class SmartRedirectHandler(urllib2.HTTPRedirectHandler):
    def http_error_301(self, req, fp, code, msg, headers):
        result = urllib2.HTTPRedirectHandler.http_error_301(self, req, fp, code, msg, headers)
        result.status = code
        return result

    def http_error_302(self, req, fp, code, msg, headers):
        result = urllib2.HTTPRedirectHandler.http_error_302(self, req, fp, code, msg, headers)
        result.status = code
        return result


class DefaultErrorHandler(urllib2.HTTPDefaultErrorHandler):
    def http_error_default(self, req, fp, code, msg, headers):
        result = urllib2.HTTPError(req.get_full_url(), code, msg, headers, fp)
        result.status = code
        return result


def open_anything(source, etag=None, lastmodified=None, agent=USER_AGENT):
    """URL, filename, or string --> stream

    This function lets you define parsers that take any input source
    (URL, pathname to local or network file, or actual data as a string)
    and deal with it in a uniform manner.  Returned object is guaranteed
    to have all the basic stdio read methods (read, readline, readlines).
    Just .close() the object when you're done with it.

    If the etag argument is supplied, it will be used as the value of an
    If-None-Match request header.

    If the lastmodified argument is supplied, it must be a formatted
    date/time string in GMT (as returned in the Last-Modified header of
    a previous request).  The formatted date/time will be used
    as the value of an If-Modified-Since request header.

    If the agent argument is supplied, it will be used as the value of a
    User-Agent request header.
    """
    if hasattr(source, 'read'):
        return source
    if source == '-':
        return sys.stdin
    if urlparse.urlparse(source)[0] == 'http':
        # Open URL with urllib2
        request = urllib2.Request(source)
        request.add_header('User-Agent', agent)
        if lastmodified:
            request.add_header('If-Modified-Since', lastmodified)
        if etag:
            request.add_header('If-None-Match', etag)
        request.add_header('Accept-encoding', 'gzip')
        opener = urllib2.build_opener(SmartRedirectHandler(), DefaultErrorHandler())
        try:
            return opener.open(request)
        except urllib2.URLError:
            return None
    # Try to open with native open function (if source is a filename)
    try:
        return open(source)
    except (IOError, OSError):
        pass
    # Treat source as string
    return StringIO(str(source))


def fetch(source, etag=None, lastmodified=None, agent=USER_AGENT, retry_attempts=FETCH_RETRY_NUM, retry_delay=FETCH_RETRY_DELAY, max_size=FETCH_MAX_SIZE):
    """Fetch data and metadata from a URL, file, stream, or string.
    """
    result = {}
    attempt_num = 0
    f = None
    while f is None and attempt_num < retry_attempts:
        if attempt_num > 0:
            sleep(retry_delay)
        attempt_num += 1
        f = open_anything(source, etag, lastmodified, agent)
    result['data'] = f.read(max_size)
    if hasattr(f, 'headers'):
        # save ETag, if the server sent one
        result['etag'] = f.headers.get('ETag')
        # save Last-Modified header, if the server sent one
        result['lastmodified'] = f.headers.get('Last-Modified')
        if f.headers.get('content-encoding') == 'gzip':
            # data came back gzip-compressed, decompress it
            result['data'] = gzip.GzipFile(fileobj=StringIO(result['data'])).read()
    if hasattr(f, 'url'):
        result['url'] = f.url
        result['status'] = 200
    if hasattr(f, 'status'):
        result['status'] = f.status
    f.close()
    return result
